==> tulip_1.2.5.orig.tar.gz: tulip-1.2.5/plugins/metric/StrengthMetric.cpp <==
#include "StrengthMetric.h"

METRICPLUGIN(StrengthMetric,"Strength","David Auber","26/02/2003","Alpha","0","1");

using namespace std;

StrengthMetric::StrengthMetric(PropertyContext *context):Metric(context) {}

StrengthMetric::~StrengthMetric() {}

double StrengthMetric::e(set<node> &U,set<node> &V) {
  set<node>::const_iterator itU;
  double result=0;
  for (itU=U.begin();itU!=U.end();++itU) {
    Iterator<node> *itN=superGraph->getInOutNodes(*itU);
    for (;itN->hasNext();) {
      node itn=itN->next();
      if (V.find(itn)!=V.end()) result+=1;
    }delete itN;
  }
  return result;
}

double StrengthMetric::e(set<node> &U) {
  set<node>::const_iterator itU;
  double result=0;
  for (itU=U.begin();itU!=U.end();++itU) {
    Iterator<node> *itN=superGraph->getInOutNodes(*itU);
    for (;itN->hasNext();) {
      node itn=itN->next();
      if (U.find(itn)!=U.end()) result+=1;
    }delete itN;
  }
  return result/2;
}

double StrengthMetric::s(set<node> &U,set<node> &V) {
  if ((U.size()==0) || (V.size()==0)) return 0;
  return (e(U,V) / (U.size()*V.size()));
}

double StrengthMetric::s(set<node> &U) {
  if (U.size()<2) return 0;
  return  (e(U)) * 2 / (U.size()*(U.size()-1));
}

double StrengthMetric::getEdgeValue(const edge e ) {
  node u=superGraph->source(e);
  node v=superGraph->target(e);
  set<node> Nu,Nv,Wuv,Mu,Mv;
  //Compute Nu
  Iterator<node> *itN=superGraph->getInOutNodes(u);
  for (;itN->hasNext();) {
    node n=itN->next();
    if (n!=v) Nu.insert(n);
  }delete itN;
  //Compute Nv
  itN=superGraph->getInOutNodes(v);
  for (;itN->hasNext();) {
    node n=itN->next();
    if (n!=u) Nv.insert(n);
  }delete itN;
  //Compute Wuv
  set<node>::const_iterator itNu;
  for (itNu=Nu.begin();itNu!=Nu.end();++itNu) {
    if (Nv.find(*itNu)!=Nv.end()) Wuv.insert(*itNu);
  }
  //Compute Mu, this implementation is not optimal, should use stl operations
  for (itNu=Nu.begin();itNu!=Nu.end();++itNu) {
    if (Wuv.find(*itNu)==Wuv.end()) Mu.insert(*itNu);
  }
  //Compute Mv, this implementation is not optimal
  set<node>::const_iterator itNv;
  for (itNv=Nv.begin();itNv!=Nv.end();++itNv) {
    if (Wuv.find(*itNv)==Wuv.end()) Mv.insert(*itNv);
  }  

  //compute strength metric
  double gamma4,gamma3;
  if ( (Mu.size() == 0) && (Mv.size()==0) && Wuv.size()==0) 
    gamma3=0;
  else
    gamma3=Wuv.size()/(Wuv.size()+Mv.size()+Mu.size());
  gamma4=s(Mu,Wuv)+s(Mv,Wuv)+s(Mu,Mv)+s(Wuv);
  
  return gamma3+gamma4;
}

double StrengthMetric::getNodeValue(const node n ) {
  //  cerr << __PRETTY_FUNCTION__ << endl;
  if (superGraph->deg(n)==0) return 0;
  double result=0;
  Iterator<edge> *itE=superGraph->getInOutEdges(n);
  for (;itE->hasNext();) {
    edge ite=itE->next();
    result+=metricProxy->getEdgeValue(ite);
  }delete itE;
  return result/superGraph->deg(n);
}

bool StrengthMetric::run() {
  return true;
}

==> tulip_2.0.0.orig.tar.gz: tulip-2.0.0/plugins/metric/StrengthMetric.cpp <==
#include "StrengthMetric.h"

METRICPLUGIN(StrengthMetric,"Strength","David Auber","26/02/2003","Alpha","0","1");

using namespace std;
using namespace stdext;

StrengthMetric::StrengthMetric(const PropertyContext &context):Metric(context) {}

StrengthMetric::~StrengthMetric() {}

double StrengthMetric::e(const hash_set<node> &U,const hash_set<node> &V) {
  hash_set<node>::const_iterator itU;
  double result=0;
  for (itU=U.begin();itU!=U.end();++itU) {
    Iterator<node> *itN=superGraph->getInOutNodes(*itU);
    while (itN->hasNext()) {
      node itn=itN->next();
      if (V.find(itn)!=V.end()) result+=1.0;
    }delete itN;
  }
  return result;
}

double StrengthMetric::e(const hash_set<node> &U) {
  hash_set<node>::const_iterator itU;
  double result=0.0;
  for (itU=U.begin();itU!=U.end();++itU) {
    Iterator<node> *itN=superGraph->getInOutNodes(*itU);
    while (itN->hasNext()) {
      node itn=itN->next();
      if (U.find(itn)!=U.end()) result+=1.0;
    }delete itN;
  }
  return result/2.0;
}

double StrengthMetric::s(const hash_set<node> &U,const hash_set<node> &V) {
  if ((U.size()==0) || (V.size()==0)) return 0;
  return (e(U,V) / double(U.size()*V.size()));
}

double StrengthMetric::s(const hash_set<node> &U) {
  if (U.size()<2) return 0.0;
  return  (e(U)) * 2.0 / double(U.size()*(U.size()-1));
}

double StrengthMetric::getEdgeValue(const edge e ) {
  node u=superGraph->source(e);
  node v=superGraph->target(e);
  hash_set<node> Nu,Nv,Wuv,Mu,Mv;
  //Compute Nu
  Iterator<node> *itN=superGraph->getInOutNodes(u);
  while (itN->hasNext()) {
    node n=itN->next();
    if (n!=v) Nu.insert(n);
  }delete itN;
  //Compute Nv
  itN=superGraph->getInOutNodes(v);
  while (itN->hasNext()) {
    node n=itN->next();
    if (n!=u) Nv.insert(n);
  }delete itN; 
  //Compute Wuv
  hash_set<node>::const_iterator itNu;
  for (itNu=Nu.begin();itNu!=Nu.end();++itNu) {
    if (Nv.find(*itNu)!=Nv.end()) Wuv.insert(*itNu);
  }
  //Compute Mu, this implementation is not optimal, should use stl operations
  for (itNu=Nu.begin();itNu!=Nu.end();++itNu) {
    if (Wuv.find(*itNu)==Wuv.end()) Mu.insert(*itNu);
  }
  //Compute Mv, this implementation is not optimal
  hash_set<node>::const_iterator itNv;
  for (itNv=Nv.begin();itNv!=Nv.end();++itNv) {
    if (Wuv.find(*itNv)==Wuv.end()) Mv.insert(*itNv);
  }  

  //compute strength metric
  double gamma4,gamma3;
  if ( (Mu.size() == 0) && (Mv.size()==0) && Wuv.size()==0) 
    gamma3=0;
  else
    gamma3=double(Wuv.size())/double((Wuv.size()+Mv.size()+Mu.size()));
  gamma4=s(Mu,Wuv)+s(Mv,Wuv)+s(Mu,Mv)+s(Wuv);
  
  return gamma3+gamma4;
}

double StrengthMetric::getNodeValue(const node n ) {
  //  cerr << __PRETTY_FUNCTION__ << endl;
  if (superGraph->deg(n)==0) return 0;
  double result=0;
  Iterator<edge> *itE=superGraph->getInOutEdges(n);
  while (itE->hasNext()) {
    edge ite=itE->next();
    result+=metricProxy->getEdgeValue(ite);
  } delete itE;
  return result/double(superGraph->deg(n));
}

bool StrengthMetric::run() {
  return true;
}
